var irc = require('irc');
var express = require('express');
var bodyParser = require('body-parser');

var config = require('./config');

var client = new irc.Client(config.server, config.nick, {
    //debug: true,
    channels: config.channels,
    userName: config.userName,
    realName: config.realName,
    retryDelay: 120000
});

var say = client.say.bind(client);

var app = express();

app.use(bodyParser.json()); // for parsing application/json
app.use(bodyParser.urlencoded({ extended: true })); // for parsing application/x-www-form-urlencoded

function formatCommit(commit) {
    return commit.message + ' ' + commit.url;
}

var handlers = {

    push: function(body) {
        var user = body.user_name;
        var projectName = body.project.name;
        var commits = body.commits;

        var msg = ['push on ' + projectName + ': ' + user + ' pushed ' + commits.length + ' commits.'];

        if (commits.length < 4) {
            msg = msg.concat(commits.map(formatCommit));
        } else {
            msg.push(formatCommit(commits[0]));
            msg.push('...');
            msg.push(formatCommit(commits[commits.length - 1]));
        }

        return msg;
    },

    issue: function(body) {
        var user = body.user.name;
        var projectName = body.project.name;

        var issue = body.object_attributes;
        var issueNumber = issue.iid;
        var issueTitle = issue.title;
        var issueState = issue.state;
        var url = issue.url;

        var msg = [projectName + ': issue #' + issueNumber + ' has changed state ("' + issueState + '")'];
        msg.push(issueTitle);
        msg.push(url);

        return msg;
    },

    merge_request: function(body) {
        var user = body.user.name;

        var request = body.object_attributes;

        var from = request.source_branch;
        var to = request.target_branch;

        var id = request.iid;
        var title = request.title;
        var url = request.url;
        var state = request.state;

        var msg = [projectName + ': merge request (' + from + ':' + to + ') #' + id + ' has changed state ("' + state + '")'];
        msg.push(title);
        msg.push(url);
    },

    build: function(body) {
        console.log('build event NYI because not documented');
        console.log(body);
        return null;
    }

};

app.post('/', function(req, res) {
    var body = req.body || {};

    var msgs = null;
    if (body.object_kind && handlers[body.object_kind])
        msgs = handlers[body.object_kind](body);

    if (msgs) {
        if (msgs instanceof Array) {
            for (var i = 0; i < msgs.length; i++)
                say(config.channels, msgs[i]);
        } else {
            say(config.channels, msgs);
        }
    }

    res.sendStatus(200);
});

app.listen(config.port, config.hostname, function() {
    console.log('gitlab-to-irc running.');
});
